#!/bin/bash
#
# Simple script to backup a GSS instance:
# database, binaries (/var/www), apache logs, users and awstats containers (if any).
# kovacs.zoltan@mimoza.hu 2005-06-09

APACHE_AUTHDIR="/etc/apache-ssl/auth.d"
APACHE_LOGDIR="/var/log/apache"
AWK="/usr/bin/awk"
AWSTATS_CONTDIR="/var/lib/awstats"
DATETIME="`date '+%Y%m%d%H%M%S'`"
DOCUMENTROOT="/var/www"
EXIT_ERR=1
GZIP="/bin/gzip"
MYSQL="/usr/bin/mysql --defaults-file=/etc/mysql/debian.cnf"
MYSQL_DUMPDB="/usr/local/sbin/mysql_dumpdb.sh"
MYSQL_DUMPTBL="/usr/local/sbin/mysql_dumptbl.sh"
SELECT_DOMAINS="SELECT hostname FROM t201_domain ORDER BY id_domain"
TAR="/bin/tar"

# Messages

MSG_MISSING="Missing file: "
MSG_NODOMAIN="No domains in "
USAGE="Usage: $0 GSS_name [MySQL_tablelist_file] as root user.\n\n"

# Must be root!

if [ $UID -ne 0 ]; then echo -ne "$USAGE" >&2; exit $EXIT_ERR; fi

# Check components

if [ ! -x "$MYSQL_DUMPDB" ];  then echo -ne "$MSG_MISSING$MYSQL_DUMPDB.\n\n"  >&2; exit $EXIT_ERR; fi
if [ ! -x "$MYSQL_DUMPTBL" ]; then echo -ne "$MSG_MISSING$MYSQL_DUMPTBL.\n\n" >&2; exit $EXIT_ERR; fi

# GSS instance is mandatory

CONTEXTROOT=$1; shift
if [ -z "$CONTEXTROOT" ]; then echo -ne "$USAGE" >&2; exit $EXIT_ERR; fi

# Database table list is optional

MYSQL_TABLELIST=$1; shift

# MySQL dump from table list (lack of them full dump)

backupfile="`pwd`/`hostname`.$DATETIME.$CONTEXTROOT.sql.gz"
if [ -f "$MYSQL_TABLELIST" ]; then
    $MYSQL_DUMPTBL -c $CONTEXTROOT $MYSQL_TABLELIST $backupfile
else
    $MYSQL_DUMPDB  -c $CONTEXTROOT $backupfile
fi

# Binary dump from DOCUMENTROOT/CONTEXTROOT

if [ ! -d $DOCUMENTROOT/$CONTEXTROOT ]; then 
    echo -ne "$MSG_MISSING$DOCUMENTROOT/$CONTEXTROOT.\n\n"  >&2; 
else
    backupfile="`pwd`/`hostname`.$DATETIME.$CONTEXTROOT.binaries.tar.gz"
    ( cd  $DOCUMENTROOT/$CONTEXTROOT ;
      $TAR 'czvf' $backupfile *
    )
fi

# Get virtualhost names from GSS instance ordered by domain number
 
backupfile="`pwd`/`hostname`.$DATETIME.$CONTEXTROOT.domains.txt"
GSSDOMAINS=`echo "$SELECT_DOMAINS" | $MYSQL -D $CONTEXTROOT 2>/dev/null | $AWK 'NR > 1'`
if [ -z "$GSSDOMAINS" ]; then echo -ne "$MSG_NODOMAIN$CONTEXTROOT.\n\n" >&2; fi
echo -e "$GSSDOMAINS" >$backupfile;

# Get Apache logs for virtualhosts

backupfile="`pwd`/`hostname`.$DATETIME.$CONTEXTROOT.apachelogs.tar"
for virtualhost in $GSSDOMAINS
do
    ( cd $APACHE_LOGDIR ;
      if [ ! -f $backupfile ]; then taroptions="cv"; else taroptions="rv"; fi
      filelist=`ls $virtualhost* 2>/dev/null`
      if [ -z "$filelist" ]; then
          echo -ne "$MSG_MISSING$APACHE_LOGDIR/$virtualhost* \n" >&2
      else
          $TAR $taroptions'f' $backupfile $virtualhost*
      fi
    )
    wait
done
if [ -f $backupfile ]; then $GZIP $backupfile; fi

# Get Apache users (statistics and DAV)

backupfile="`pwd`/`hostname`.$DATETIME.$CONTEXTROOT.apacheusers.tar"
( cd $APACHE_AUTHDIR ;
  if [ ! -f "$CONTEXTROOT.group" ]; then
      echo -ne "$MSG_MISSING$APACHE_AUTHDIR/$CONTEXTROOT.group \n" >&2
  else
      if [ ! -f $backupfile ]; then taroptions="cv"; else taroptions="rv"; fi
      $TAR $taroptions'f' $backupfile $CONTEXTROOT.group
  fi
)
( cd $APACHE_AUTHDIR ;
  if [ ! -f "$CONTEXTROOT.user" ]; then
      echo -ne "$MSG_MISSING$APACHE_AUTHDIR/$CONTEXTROOT.user \n" >&2
  else
      if [ ! -f $backupfile ]; then taroptions="cv"; else taroptions="rv"; fi
      $TAR $taroptions'f' $backupfile $CONTEXTROOT.user
  fi
)

for virtualhost in $GSSDOMAINS
do
    ( cd $APACHE_AUTHDIR ;
      if [ ! -f "$virtualhost.group" ]; then
          echo -ne "$MSG_MISSING$APACHE_AUTHDIR/$virtualhost.group \n" >&2
      else
          if [ ! -f $backupfile ]; then taroptions="cv"; else taroptions="rv"; fi
          $TAR $taroptions'f' $backupfile $virtualhost.group
      fi
    )
    wait
    ( cd $APACHE_AUTHDIR ;
      if [ ! -f "$virtualhost.user" ]; then
          echo -ne "$MSG_MISSING$APACHE_AUTHDIR/$virtualhost.user \n" >&2
      else
          if [ ! -f $backupfile ]; then taroptions="cv"; else taroptions="rv"; fi
          $TAR $taroptions'f' $backupfile $virtualhost.user
      fi
    )
    wait
done
if [ -f $backupfile ]; then $GZIP $backupfile; fi

# Get AwStats containerss for virtualhosts

backupfile="`pwd`/`hostname`.$DATETIME.$CONTEXTROOT.awstats.tar"
for virtualhost in $GSSDOMAINS
do
    ( cd $AWSTATS_CONTDIR ;
      if [ ! -f $backupfile ]; then taroptions="cv"; else taroptions="rv"; fi
      filelist=`ls *$virtualhost* 2>/dev/null`
      if [ -z "$filelist" ]; then
          echo -ne "$MSG_MISSING$AWSTATS_CONTDIR/*$virtualhost* \n" >&2
      else
          $TAR $taroptions'f' $backupfile *$virtualhost*
      fi
    )
    wait
done
if [ -f $backupfile ]; then $GZIP $backupfile; fi

